package Collections

/**
 * https://www.tutorialspoint.com/kotlin/kotlin_lists.htm
 */
class Lists {

	/**
	 * How to instantiate a mutable and immutable list of strings.
	 * Output is the same for both.
	 */
	fun printMutableImmutable() {
		val theList = listOf("one", "two", "three", "four")
		println(theList)
		val theMutableList = mutableListOf("one", "two", "three", "four")
		println(theMutableList)
	}

	/**
	 * toString() just produces the same string as direct printing would
	 */
	fun listToString() {
		val theList = listOf("one", "two", "three", "four")
		println(theList.toString())
	}

	/**
	 * How to iterate over a list with a while loop
	 */
	fun iterateOverListWhile() {
		val theList = listOf("one", "two", "three", "four")
		val itr = theList.listIterator()
		while (itr.hasNext()) {
			println(itr.next())
		}
	}

	/**
	 * How to iterate over a list using a for loop
	 */
	fun iterateOverListFor() {
		val theList = listOf("one", "two", "three", "four")
		for (i in theList.indices) {
			println(theList[i])
		}
	}

	/**
	 * How to iterate over a list using a forEach loop
	 */
	fun iterateOverListForEach() {
		val theList = listOf("one", "two", "three", "four")
		theList.forEach {
			println(it)
		}
	}

	/**
	 * How to get the number of elements in a list
	 */
	fun getListSize() {
		val theList = listOf("one", "two", null, "four", "five")
		println("Size of the list " + theList.size)
	}

	/**
	 * Check, if an element is present in a list
	 */
	fun checkElementInList() {
		val theList = listOf("one", "two", "three", "four")
		if ("two" in theList) {
			println(true)
		} else {
			println(false)
		}
	}

	/**
	 * Another way of checking the presence of an element in a list
	 */
	fun checkElementInList2() {
		val theList = listOf("one", "two", "three", "four")
		if (theList.contains("two")) {
			println(true)
		} else {
			println(false)
		}
	}

	/**
	 * How to check if a list is empty
	 */
	fun checkEmptyList() {
		val theList = listOf("one", "two", "three", "four")
		if (theList.isEmpty()) {
			println(true)
		} else {
			println(false)
		}
	}

	/**
	 * How to get the index of an element in a list
	 */
	fun getIndexOfElement() {
		val theList = listOf("one", "two", "three", "four")
		println("Index of 'two' : " + theList.indexOf("two")) // 1
		println("Index of 'five' : " + theList.indexOf("five")) // -1
	}

	/**
	 * Get the element at the specified index of the list
	 */
	fun getElementAtIndex() {
		val theList = listOf("one", "two", "three", "four")
		println("Element at 3rd position " + theList.get(2))
	}

	/**
	 * How to combine to lists
	 */
	fun combineTwoLists() {
		val firstList = listOf("one", "two", "three")
		val secondList = listOf("four", "five", "six")
		val resultList = firstList + secondList
		println(resultList)
	}

	/**
	 * How to remove all elements of one list from another, if they are present
	 */
	fun removeListFromList() {
		val firstList = listOf("one", "two", "three")
		val secondList = listOf("one", "five", "six")
		val resultList = firstList - secondList
		println(resultList) // [two, three]
	}

	/**
	 * Slices the list, i.e. creates a sublist
	 */
	fun sliceList() {
		val theList = listOf("one", "two", "three", "four", "five")
		val resultList = theList.slice(2..4)
		println(resultList) // [three, four, five]
	}

	/**
	 * How to filter null values out of a given list
	 */
	fun filterNullValues() {
		val theList = listOf("one", "two", null, "four", "five")
		val resultList = theList.filterNotNull()
		println(resultList)
	}

	/**
	 * Filters all elements from a list that do not meet a defined condition
	 */
	fun filterElementsBasedOnCondition() {
		val theList = listOf(10, 20, 30, 31, 40, 50, -1, 0)
		val resultList = theList.filter { it > 30 }
		println(resultList) // [31, 40, 50]
	}

	/**
	 * Drops the first n elements from the list
	 */
	fun dropFirstnElements() {
		val theList = listOf(10, 20, 30, 31, 40, 50, -1, 0)
		val resultList = theList.drop(3)
		println(resultList) // [31, 40, 50, -1, 0]
	}

	/**
	 * Groups the elements of a list based on the given grouping criteria
	 */
	fun groupElementsInList() {
		val theList = listOf(10, 12, 30, 31, 40, 9, -3, 0)
		val resultList = theList.groupBy { it % 3 }
		println(resultList) // {1=[10, 31, 40], 0=[12, 30, 9, -3, 0]}
	}

	/**
	 * Maps every element of the list to another value based on the given transformation
	 */
	fun transformElementsInList() {
		val theList = listOf(10, 12, 30, 31, 40, 9, -3, 0)
		val resultList = theList.map { it / 3 }
		println(resultList)
	}

	/**
	 * Creates chunks of the given size
	 */
	fun createListChunks() {
		val theList = listOf(10, 12, 30, 31, 40, 9, -3, 0)
		val resultList = theList.chunked(3)
		println(resultList) // [[10, 12, 30], [31, 40, 9], [-3, 0]]
	}

	/**
	 * Creates snapshots of the list of the given size.
	 */
	fun createListSnapshots() {
		val theList = listOf(10, 12, 30, 31, 40, 9, -3, 0)
		val resultList = theList.windowed(3)
		println(resultList) // [[10, 12, 30], [12, 30, 31], [30, 31, 40], [31, 40, 9], [40, 9, -3], [9, -3, 0]]
	}

	/**
	 * Creates snapshots of the list of the given size, using the given step size
	 */
	fun createSteppedListSnapshots() {
		val theList = listOf(10, 12, 30, 31, 40, 9, -3, 0)
		val resultList = theList.windowed(3, 3)
		println(resultList) // [[10, 12, 30], [31, 40, 9]]
	}

	/**
	 * How to add and remove elements from a mutable List
	 */
	fun addAndRemoveElements() {
		val theList = mutableListOf(
			10,
			20,
			30
		)
		theList.add(40)
		theList.add(50)
		println(theList) // [10, 20, 30, 40, 50]
		theList.remove(10)
		theList.remove(30)
		println(theList) // [20, 40, 50]
	}
}
