package Collections

/**
 * https://www.tutorialspoint.com/kotlin/kotlin_maps.htm
 */
class Maps {
	/**
	 * How to instantiate mutable and immutable maps
	 */
	fun instantiateMaps() {
		val theMap = mapOf(
			"one" to 1,
			"two" to 2,
			"three" to 3,
			"four" to 4
		)
		println(theMap) // {one=1, two=2, three=3, four=4}
		val theMutableMap = mutableMapOf(
			"one" to 1,
			"two" to 2,
			"three" to 3,
			"four" to 4
		)
		println(theMutableMap) // {one=1, two=2, three=3, four=4}
	}

	/**
	 * How to add new elements to a map
	 */
	fun addElementsToMap() {
		val theMap = HashMap<String, Int>()
		theMap["one"] = 1
		theMap["two"] = 2
		theMap["three"] = 3
		theMap["four"] = 4
		println(theMap) // {four=4, one=1, two=2, three=3}
	}

	/**
	 * Another way of instantiating a map
	 */
	fun instantiateMap2() {
		val theMap = mapOf(
			Pair("one", 1),
			Pair("two", 2),
			Pair("three", 3)
		)
		println(theMap)
	}

	/**
	 * How to return entries / keys / values from a map
	 */
	fun getEntriesKeysValues() {
		val theMap = mapOf(
			"one" to 1,
			"two" to 2,
			"three" to 3,
			"four" to 4
		)
		println("Entries: " + theMap.entries) // [one=1, two=2, three=3, four=4]
		println("Keys: " + theMap.keys) // [one, two, three, four]
		println("Values: " + theMap.values) // [1, 2, 3, 4]
	}

	/**
	 * Map toString method returns the same string as a print would do
	 */
	fun mapToString() {
		val theMap = mapOf("one" to 1, "two" to 2, "three" to 3, "four" to 4)
		println(theMap.toString())
	}

	/**
	 * How to iterate over a map using an iterator
	 */
	fun iterateWithIterator() {
		val theMap = mapOf("one" to 1, "two" to 2, "three" to 3, "four" to 4)
		val itr = theMap.keys.iterator()
		while (itr.hasNext()) {
			val key = itr.next()
			val value = theMap[key]
			println("${key}=$value")
		}
	}

	/**
	 * How to iterate over keys and values using a for ... in loop
	 */
	fun iterateWithForLoop() {
		val theMap = mapOf("one" to 1, "two" to 2, "three" to 3, "four" to 4)
		for ((k, v) in theMap) {
			println("$k = $v")
		}
	}

	/**
	 * How to iterate over keys and values using a forEach loop
	 */
	fun iterateWithForEach() {
		val theMap = mapOf(
			"one" to 1,
			"two" to 2,
			"three" to 3,
			"four" to 4
		)
		theMap.forEach { k, v ->
			println("Key = $k, Value = $v")
		}
	}

	/**
	 * How to get the size of the map.
	 * Both ways return the same value
	 */
	fun getMapSize() {
		val theMap = mapOf(
			"one" to 1,
			"two" to 2,
			"three" to 3,
			"four" to 4
		)
		println("Size of the Map: " + theMap.size)
		println("Size of the Map: " + theMap.count())
	}

	/**
	 * How to check if a specific key or value is present in a map
	 */
	fun checkIfMapContainsKeyOrValue() {
		val theMap = mapOf("one" to 1, "two" to 2, "three" to 3, "four" to 4)
		if (theMap.containsKey("two")) {
			println(true)
		} else {
			println(false)
		}
		if (theMap.containsValue(2)) {
			println(true)
		} else {
			println(false)
		}
	}

	/**
	 * How to check if a map is empty
	 */
	fun checkIfMapIsEmpty() {
		val theMap = mapOf("one" to 1, "two" to 2, "three" to 3, "four" to 4)
		if (theMap.isEmpty()) {
			println(true)
		} else {
			println(false)
		}
	}

	/**
	 * How to access an element in a map by its key
	 */
	fun accessElementsInMap() {
		val theMap = mapOf(
			"one" to 1,
			"two" to 2,
			"three" to 3,
			"four" to 4
		)
		println("The value for key two " + theMap.get("two"))
		println("The value for key two " + theMap["two"]) // Preferred way
	}

	/**
	 * How to combine two maps into a new map
	 */
	fun combineTwoMaps() {
		val firstMap = mapOf("one" to 1, "two" to 2, "three" to 3)
		val secondMap = mapOf("one" to 10, "four" to 4)
		val resultMap = firstMap + secondMap
		println(resultMap) // {one=10, two=2, three=3, four=4}
	}

	/**
	 * How to remove all elements with given keys from a map
	 */
	fun removeListOfKeysFromMap() {
		val theMap = mapOf("one" to 1, "two" to 2, "three" to 3)
		val theKeyList = listOf("one", "four")
		val resultMap = theMap - theKeyList
		println(resultMap) // {two=2, three=3}
	}

	/**
	 * Another way of removing elements from a map
	 */
	fun removeElementsFromMap() {
		val theMap = mutableMapOf(
			"one" to 1,
			"two" to 2,
			"three" to 3,
			"four" to 4
		)
		theMap.remove("two")
		println(theMap) // {one=1, three=3, four=4}
		theMap -= listOf("three")
		println(theMap) // {one=1, four=4}
	}

	/**
	 * How to sort a map
	 */
	fun sortMap() {
		val theMap = mapOf("one" to 1, "two" to 2, "three" to 3, "four" to 4)
		val resultMap = theMap.toSortedMap()
		println(resultMap) // {four=4, one=1, three=3, two=2}
	}

	/**
	 * How to filter elements from a map
	 */
	fun filterMap() {
		val theMap = mapOf("one" to 1, "two" to 2, "three" to 3, "four" to 4)
		var resultMap = theMap.filterValues { it > 2 }
		println(resultMap) // {three=3, four=4}
		resultMap = theMap.filterKeys { it == "two" }
		println(resultMap) // {two=2}
		resultMap = theMap.filter { it.key == "two" || it.value == 4 }
		println(resultMap) // {two=2, four=4}
	}

	/**
	 * How to map a map to another map using a sort-of lambda expression
	 */
	fun mapLambdaExpression() {
		val theMap = mapOf("one" to 1, "two" to 2, "three" to 3)
		val resultMap = theMap.map { (k, v) ->
			"Key is $k, Value is $v"
		}
		println(resultMap) // [Key is one, Value is 1, Key is two, Value is 2, Key is three, Value is 3]
	}

	/**
	 * How to put elements into a map and remove them
	 */
	fun addAndRemoveElements() {
		val theMap = mutableMapOf(
			"one" to 1,
			"two" to 2,
			"three" to 3,
			"four" to 4
		)
		theMap.put("four", 4)
		println(theMap)
		theMap["five"] = 5
		println(theMap)
		theMap.remove("two")
		println(theMap)
	}
}
