package Collections

/**
 * https://www.tutorialspoint.com/kotlin/kotlin_sets.htm
 */
class Sets {
	/**
	 * How to instantiate mutable an immutable Sets
	 */
	fun instantiateSets() {
		val theSet = setOf("one", "two", "three", "four")
		println(theSet) // [one, two, three, four]
		val theMutableSet = mutableSetOf("one", "two", "three", "four")
		println(theMutableSet) // [one, two, three, four]
	}

	/**
	 * How to print a set
	 */
	fun printSet() {
		val theSet = setOf("one", "two", "three", "four")
		println(theSet.toString()) // [one, two, three, four]
	}

	/**
	 * How to iterate over a set using an iterator
	 */
	fun iterateOverSetWithIterator() {
		val theSet = setOf("one", "two", "three", "four")
		val itr = theSet.asIterable().iterator()
		while (itr.hasNext()) {
			println(itr.next())
		}
	}

	/**
	 * How to iterate over the set indices
	 */
	fun iterateOverSetIndices() {
		val theSet = setOf("one", "two", "three", "four")
		for (i in theSet.indices) {
			println(theSet.elementAt(i))
		}
	}

	/**
	 * How to iterate over a set using forEach
	 */
	fun iterateOverSetForEach() {
		val theSet = setOf("one", "two", "three", "four")
		theSet.forEach {
			println(it)
		}
	}

	/**
	 * How to get the size of a set
	 */
	fun getSetSize() {
		val theSet = setOf("one", "two", null, "four", "five")
		println("Size of the Set " + theSet.size)
	}

	/**
	 * How to check if an element is present in a set
	 */
	fun checkElementPresentInSet() {
		val theSet = setOf("one", "two", "three", "four")
		if ("two" in theSet) {
			println(true)
		} else {
			println(false)
		}
	}

	/**
	 * Another way of checking if an element is present in a set
	 */
	fun checkElementPresentInSet2() {
		val theSet = setOf("one", "two", "three", "four")
		if (theSet.contains("two")) {
			println(true)
		} else {
			println(false)
		}
	}

	/**
	 * How to check if a set is empty
	 */
	fun checkIfSetIsEmpty() {
		val theSet = setOf("one", "two", "three", "four")
		if (theSet.isEmpty()) {
			println(true)
		} else {
			println(false)
		}
	}

	/**
	 * How to get the index of an element in the set
	 */
	fun getIndexOfElement() {
		val theSet = setOf("one", "two", "three", "four")
		println("Index of 'two' - " + theSet.indexOf("two"))
	}

	/**
	 * How to get the element at a given index
	 */
	fun getElementAtIndex() {
		val theSet = setOf("one", "two", "three", "four")
		println("Element at 3rd position " + theSet.elementAt(2))
	}

	/**
	 * How to combine two sets
	 */
	fun combineTwoSets() {
		val firstSet = setOf("one", "two", "three")
		val secondSet = setOf("one", "four", "five", "six")
		val resultSet = firstSet + secondSet
		println(resultSet)
	}

	/**
	 * How to remove a set of elements from another set
	 */
	fun removeSetFromSet() {
		val firstSet = setOf("one", "two", "three")
		val secondSet = setOf("one", "five", "six")
		val resultSet = firstSet - secondSet
		println(resultSet) // [two, three]
	}

	/**
	 * How to filter null values out of a set
	 */
	fun filterNullValues() {
		val theSet = setOf("one", "two", null, "four", "five")
		val resultSet = theSet.filterNotNull()
		println(resultSet)
	}

	/**
	 * How to sort elements in a set.
	 * As sets are, per their nature, unsorted, these methods return a list instead of a set
	 */
	fun sortSetOfElements() {
		val theSet = setOf(10, 20, 30, 31, 40, 50, -1, 0)
		var resultSet = theSet.sorted()
		println(resultSet) // [-1, 0, 10, 20, 30, 31, 40, 50]
		resultSet = theSet.sortedDescending()
		println(resultSet) // [50, 40, 31, 30, 20, 10, 0, -1]
		println(resultSet::class.java.typeName)
	}

	/**
	 * Filters the set for elements that do not meet certain criteria
	 */
	fun filterElementsInSet() {
		val theSet = setOf(10, 20, 30, 31, 40, 50, -1, 0)
		val resultSet = theSet.filter { it > 30 }
		println(resultSet) // [31, 40, 50]
	}

	/**
	 * How to drop the first n elements
	 */
	fun dropFirstnElements() {
		val theSet = setOf(10, 20, 30, 31, 40, 50, -1, 0)
		val resultSet = theSet.drop(3)
		println(resultSet) // [31, 40, 50, -1, 0]
	}

	/**
	 * How to group elements of a set based in specific criteria.
	 * Returns a map of elements.
	 */
	fun groupElementsInSet() {
		val theSet = setOf(10, 12, 30, 31, 40, 9, -3, 0)
		val resultSet = theSet.groupBy { it % 3 }
		println(resultSet)
	}

	/**
	 * Map every element of a set based on the given mapping function.
	 * Returns a list of elements
	 */
	fun lambdaMapElements() {
		val theSet = setOf(10, 12, 30, 31, 40, 9, -3, 0)
		val resultSet = theSet.map { it / 3 }
		println(resultSet)
	}

	/**
	 * Divides the set into chunks of the given size.
	 * Returns a list of lists.
	 */
	fun divideSetIntoChunks() {
		val theSet = setOf(10, 12, 30, 31, 40, 9, -3, 0)
		val resultSet = theSet.chunked(3)
		println(resultSet) // [[10, 12, 30], [31, 40, 9], [-3, 0]]
	}

	/**
	 * Creates snapshots of the set of the given size.
	 */
	fun createSetSnapshots() {
		val theSet = setOf(10, 12, 30, 31, 40, 9, -3, 0)
		val resultSet = theSet.windowed(3)
		println(resultSet)
	}

	/**
	 * Creates snapshots of the set of the given size, using the given step size
	 */
	fun createSteppedSetSnapshots() {
		val theSet = setOf(10, 12, 30, 31, 40, 9, -3, 0)
		val resultSet = theSet.windowed(3, 3)
		println(resultSet)
	}

	/**
	 * How to add and remove elements from / to the set
	 */
	fun addAndRemoveElements() {
		val theSet = mutableSetOf(
			10,
			20,
			30
		)
		theSet.add(40)
		theSet.add(50)
		println(theSet) // [10, 20, 30, 40, 50]
		theSet.remove(10)
		theSet.remove(30)
		println(theSet) // [20, 40, 50]
	}
}
