package ControlFlow

/**
 * https://www.tutorialspoint.com/kotlin/kotlin_when_expression.htm
 */
class WhenStatements {
	/**
	 * Just like a regular switch case expression that returns a value
	 */
	fun switchCaseReturn() {
		val day = 2
		val result = when (day) {
			1 -> "Monday"
			2 -> "Tuesday"
			3 -> "Wednesday"
			4 -> "Thursday"
			5 -> "Friday"
			6 -> "Saturday"
			7 -> "Sunday"
			else -> "Invalid day."
		}
		println(result)
	}

	/**
	 * Switch case, but with code executed for each case
	 */
	fun switchCaseCodeBlocks() {
		val day = 2
		when (day) {
			1 -> println("Monday")
			2 -> println("Tuesday")
			3 -> println("Wednesday")
			4 -> println("Thursday")
			5 -> println("Friday")
			6 -> println("Saturday")
			7 -> println("Sunday")
			else -> println("Invalid day.")
		}
	}

	/**
	 * Switch case but multiple cases result in the same code being executed
	 */
	fun switchCaseMultipleCases() {
		val day = 2
		when (day) {
			1, 2, 3, 4, 5 -> println("Weekday")
			else -> println("Weekend")
		}
	}

	/**
	 * Switch case but the case is with a range
	 */
	fun switchCaseWithRange() {
		val day = 2
		when (day) {
			in 1..5 -> println("Weekday")
			else -> println("Weekend")
		}
	}

	/**
	 * Use advanced mathematic shit to distinguish the relevant case
	 */
	fun switchCaseWithAdvancedCaseShit() {
		val x = 20
		val y = 10
		val z = 10
		when (x) {
			(y + z) -> print("y + z = x = $x")
			else -> print("Condition is not satisfied")
		}
	}

	/**
	 * Executes a whole block of code for each case
	 */
	fun switchCaseWithMoreCodePerBlock() {
		val day = 2
		when (day) {
			1 -> {
				println("First day of the week")
				println("Monday")
			}
			2 -> {
				println("Second day of the week")
				println("Tuesday")
			}
			3 -> {
				println("Third day of the week")
				println("It is Wednesday, my dudes!")
			}
			4 -> println("Thursday")
			5 -> println("Friday")
			6 -> println("Saturday")
			7 -> println("Sunday")
			else -> println("Invalid day.")
		}
	}

}
